﻿using System;
using Microsoft.Practices.Prism.Regions;
using Telerik.Windows.Controls;
using System.Windows.Controls;
using System.Windows;
using System.ComponentModel.Composition.Hosting;
using System.ComponentModel.Composition;

namespace Northwind.Common
{
    [Export]
    public class QATBRegionAdapter : RegionAdapterBase<QuickAccessToolBar>        
    {
        private QuickAccessToolBar _instance;
        private IRegionManager _regionManager;

        /// <summary>
        /// <see cref="Style"/> to set to the created <see cref="TabItem"/>.
        /// </summary>
        public static readonly DependencyProperty ItemContainerStyleProperty =
            DependencyProperty.RegisterAttached("ItemContainerStyle", typeof(Style), typeof(TabControlRegionAdapter), null);

                /// <summary>
        /// Initializes a new instance of <see cref="ItemsControlRegionAdapter"/>.
        /// </summary>
        /// <param name="regionBehaviorFactory">The factory used to create the region behaviors to attach to the created regions.</param>
        [ImportingConstructor]
        public QATBRegionAdapter(IRegionBehaviorFactory regionBehaviorFactory, IRegionManager regionManager)
            : base(regionBehaviorFactory)
        {
            _regionManager = regionManager;
        }

        /// <summary>
        /// Gets the <see cref="ItemContainerStyleProperty"/> property value.
        /// </summary>
        /// <param name="target">Target object of the attached property.</param>
        /// <returns>Value of the <see cref="ItemContainerStyleProperty"/> property.</returns>
        public static Style GetItemContainerStyle(DependencyObject target)
        {
            if (target == null) throw new ArgumentNullException("target");
            return (Style)target.GetValue(ItemContainerStyleProperty);
        }

        /// <summary>
        /// Sets the <see cref="ItemContainerStyleProperty"/> property value.
        /// </summary>
        /// <param name="target">Target object of the attached property.</param>
        /// <param name="value">Value to be set on the <see cref="ItemContainerStyleProperty"/> property.</param>
        public static void SetItemContainerStyle(DependencyObject target, Style value)
        {
            if (target == null) throw new ArgumentNullException("target");
            target.SetValue(ItemContainerStyleProperty, value);
        }

        protected override void Adapt(IRegion region, QuickAccessToolBar regionTarget)
        {
            _instance = regionTarget;
            _instance.Items.Clear();

            if (region == null) throw new ArgumentNullException("region");
            if (regionTarget == null) throw new ArgumentNullException("regionTarget");

            bool itemsSourceIsSet = regionTarget.ItemsSource != null;
#if !SILVERLIGHT
            itemsSourceIsSet = itemsSourceIsSet || (BindingOperations.GetBinding(regionTarget, ItemsControl.ItemsSourceProperty) != null);
#endif
            if (itemsSourceIsSet)
            {
                throw new InvalidOperationException("ItemsSource on QATB passed to QATBRegionAdapter is null.");
            }

            // If control has child items, move them to the region and then bind control to region. Can't set ItemsSource if child items exist.
            if (regionTarget.Items.Count > 0)
            {
                foreach (object childItem in regionTarget.Items)
                {
                    region.Add(childItem);
                }
                // Control must be empty before setting ItemsSource
                regionTarget.Items.Clear();
            }

            regionTarget.ItemsSource = region.Views;


            if (!_regionManager.Regions.ContainsRegionWithName(region.Name))
            {
                _regionManager.Regions.Add(region);
            }

        }


        protected override IRegion CreateRegion()
        {
            return new Region();
        }
    }
}
