﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Windows.Media.Imaging;

namespace Northwind.Common
{
    public partial class Marker : UserControl
    {
        const string ImagesURI = "/Northwind.Common;Component/Assets/Images/";
        
        public Marker()
        {
            InitializeComponent();

            this.Loaded += new System.Windows.RoutedEventHandler(this.Marker_Loaded);
            this.Unloaded += new RoutedEventHandler(Marker_Unloaded);
        }

        void Marker_Unloaded(object sender, RoutedEventArgs e)
        {
            this.Loaded -= new System.Windows.RoutedEventHandler(this.Marker_Loaded);
            this.Unloaded -= new RoutedEventHandler(Marker_Unloaded);
        }

        void Marker_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {
            this.SetupMarker();
        }

        public void SetupMarker()
        {
            try
            {
                MarkerModel data = this.DataContext as MarkerModel;
                if (data != null && data.Coordinates != null)
                {
                    //Clear
                    LayoutRoot.Children.Clear();

                    //Add
                    switch (data.Type)
                    {
                        case Enums.MarkerType.Alarm:
                            LayoutRoot.Children.Add(new TransactionMarker(data.Index.ToString(), string.Concat(ImagesURI, "RedMarker.png")));
                            break;
                        case Enums.MarkerType.Alert:
                            LayoutRoot.Children.Add(new TransactionMarker(data.Index.ToString(), string.Concat(ImagesURI, "YellowMarker.png")));
                            break;
                        case Enums.MarkerType.Notice:
                            LayoutRoot.Children.Add(new TransactionMarker(data.Index.ToString(), string.Concat(ImagesURI, "LightBlueMarker.png")));
                            break;
                        case Enums.MarkerType.Current:
                            LayoutRoot.Children.Add(new OrbMarker());
                            break;
                        case Enums.MarkerType.Point:                        
                            LayoutRoot.Children.Add(new PointOrbMarker(data.Fill, data.Stroke, data.LabelData));
                            break;
                        case Enums.MarkerType.Transaction:
                            LayoutRoot.Children.Add(new TransactionMarker(data.Index.ToString(), string.Concat(ImagesURI, "BlackMarker.png")));
                            break;

                        case Enums.MarkerType.MultiPoint:
                            LayoutRoot.Children.Add(new MultiPointMarker("~", string.Concat(ImagesURI, "MultiPoint.png")));

                            break;
                    }


                    if(data.Type == Enums.MarkerType.MultiPoint)
                    {
                        CoordinatesTextBlock.Text = "Multi items at this coordinates.";
                    }
                    else
                    {
                        if (data.Coordinates.TimeStamp.HasValue)
                        {
                            TimeStampTextBlock.Text = data.Coordinates.TimeStamp.Value.ToShortTimeString();
                        }

                        if (data.Coordinates.Lat.HasValue && data.Coordinates.Lat.Value != 0)
                            CoordinatesTextBlock.Text = string.Format("{0}, {1}", SubStringCoords(data.Coordinates.Lat.ToString(), 0, 8), SubStringCoords(data.Coordinates.Lon.ToString(), 0, 8));
                    }
                }
            }
            catch (System.Exception e)
            {
                throw new Exception(this.GetType().ToString(), e);
            }
        }

        private void BuildImageControl(string imageFileName)
        {
            try
            {
                Image myImage = new Image();
                myImage.Source = new BitmapImage(new Uri(string.Concat(ImagesURI, imageFileName), UriKind.RelativeOrAbsolute));
                LayoutRoot.Children.Add(myImage);
            }
            catch (System.Exception e)
            {

            }
        }

        /// <summary>
        /// The purpose of this function is to handle a bug where Setmarkers was hard coded to substring to 8 but some coordinates didn't have 8 chars.
        /// This function will allow you to pass in the index to start and the Length to substring and if the string doesn't have => chars of the length,
        /// then it just returns
        /// </summary>
        /// <param name="coord"></param>
        /// <param name="startIndex"></param>
        /// <param name="lenghth"></param>
        /// <returns></returns>
        private string SubStringCoords(string coord, int startIndex, int lenghth)
        {
            if (coord.Length > 8)
                return coord.Substring(0, 8);
            else
                return coord;

        }
    }
}
