﻿using System;
using System.ComponentModel.Composition;
using Microsoft.Practices.Prism.Regions;
using System.Windows.Controls;
using GalaSoft.MvvmLight;

namespace Northwind.Common
{
    [Export(typeof(IPopupController))]
    public class PopupController : IPopupController
    {
        #region Private Fields

        IRegionManager _regionManager;
        private IRegion _mainPopupRegion;

        #endregion

        #region ctor

        [ImportingConstructor]
        public PopupController(IRegionManager regionManager)
        {
            _regionManager = regionManager;

        }

        #endregion

        // Call if you need to add your own command to close a popup window
        public void CloseMainPopup(bool cleanUp)
        {
            if (_mainPopupRegion == null)
                _mainPopupRegion = _regionManager.Regions[GlobalRegionNames.PopupMainRegion];

            // Clear any Views in the PopupRegion
            object _currentView = _mainPopupRegion.GetView(GlobalRegionNames.PopupMainRegion);

            if (_currentView != null)
            {
                // This will close the popup
                _mainPopupRegion.Deactivate(_currentView);
                // Remove the view from the popup
                _mainPopupRegion.Remove(_currentView);

                // Clean up the View if caller requests to do so
                if (cleanUp)
                {
                    ((ICleanup)_currentView).Cleanup();
                    _currentView = null;
                }
            }
        }

        // Shows the view passed in a popup window
        public void ShowMainPopup(UserControl view)
        {
            if (_mainPopupRegion == null)
                _mainPopupRegion = _regionManager.Regions[GlobalRegionNames.PopupMainRegion];

            // Clear any Views in the PopupRegion
            object _currentView = _mainPopupRegion.GetView(GlobalRegionNames.PopupMainRegion);

            if (_currentView != null)
                _mainPopupRegion.Remove(_currentView);

            _mainPopupRegion.Add(view, GlobalRegionNames.PopupMainRegion);
            _mainPopupRegion.Activate(view);
        }
    }
}
