﻿using System;
using System.Linq;
using Telerik.Windows.Controls.Map;
using System.Collections.Generic;
using System.Windows.Media;

namespace Northwind.Common
{
    public class MapHelpers
    {
        // This method attempts to manually calculate center and Zoom.  Center works good, but zoom 
        // is going to rely on size elements from the map which I haven't figured out how to do with Telerik
        public static CenterAndZoomModel CalculateMapCenterZoom(IList<MarkerModel> markers, MapZoomLevelModel _mapZoomLevels, DistanceUnit distanceUnit)
        {
            // Note I could have used lists here but I didn't want a field having a bad value such as 0 in
            // the collection passed in to throw off the center.  So I loop through and get the highest and lowest Lat & Lon values
            // Add them and divide by 2 for now.
            CenterAndZoomModel _centerAndZoom = new CenterAndZoomModel();

            // MinLat & Lons would represent the SouthWest corner of a LocationRect
            // MaxLat & Lons would represent the NorthEast corner of a LocationRect

            double _minLat = 0;
            double _maxLat = 0;
            
            double _minLon = 0;
            double _maxLon = 0;

            double _distValue = 0;

            // Use Kilometers or Miles
            if (distanceUnit == DistanceUnit.Kilometer)
            {
                _distValue = 6371;
            }
            else
            {
                _distValue = 3958.75;
            }

            // Assign our vars initial values
            if (markers != null && markers.Count > 0)
            {
                _minLat = markers[0].Coordinates.Lat.Value;
                _maxLat = markers[0].Coordinates.Lat.Value;

                _minLon = markers[0].Coordinates.Lon.Value;
                _maxLon = markers[0].Coordinates.Lon.Value;
            }

            foreach (var m in markers)
            {
                if (m.Coordinates != null && m.Coordinates.Lat.HasValue && m.Coordinates.Lon.HasValue)
                {                    
                    if (m.Coordinates.Lat.Value < _minLat)
                        _minLat = m.Coordinates.Lat.Value;

                    if (m.Coordinates.Lat.Value > _maxLat)
                        _maxLat = m.Coordinates.Lat.Value;

                    if (m.Coordinates.Lon.Value < _minLon)
                        _minLon = m.Coordinates.Lon.Value;

                    if (m.Coordinates.Lon.Value > _maxLon)
                        _maxLon = m.Coordinates.Lon.Value;
                }
            }

            if (markers.Count > 0)
            {
                // Populate Coordinates
                double _totalLatVals = _minLat + _maxLat;
                double _totalLonVals = _minLon + _maxLon;

                Location _latLon = new Location();

                _latLon.Latitude = _totalLatVals != 0 ? _totalLatVals / 2 : 0;
                _latLon.Longitude = _totalLonVals != 0 ? _totalLonVals / 2 : 0;

                _centerAndZoom.MapCenter = _latLon;

                // Calculate Zoom
                // From Aiska Hendra
                // http://www.codedblog.com/2007/09/28/how-to-auto-zoom-and-auto-center-google-maps/
                // 6371 would be in kilometers
                //var miles = (6371 * Math.Acos(Math.Sin(_lowestLatVal / 57.2958) * Math.Sin(_highestLatVal / 57.2958) + Math.Cos(_lowestLatVal / 57.2958) *
                //                Math.Cos(_highestLatVal / 57.2958) * Math.Cos(_highestLonVal / 57.2958 - _lowestLonVal / 57.2958)));
                // This would calculate miles.
                //var miles = (3958.75 * Math.Acos(Math.Sin(_minLat / 57.2958) * Math.Sin(_maxLat / 57.2958) + Math.Cos(_minLat / 57.2958) *
                //                Math.Cos(_maxLat / 57.2958) * Math.Cos((_maxLon / 57.2958) - (_minLon / 57.2958))));
                var _dist = (_distValue * Math.Acos(Math.Sin(_minLat / 57.2958) * Math.Sin(_maxLat / 57.2958) + Math.Cos(_minLat / 57.2958) *
                                Math.Cos(_maxLat / 57.2958) * Math.Cos((_maxLon / 57.2958) - (_minLon / 57.2958))));


                double _mapDisplay = 200;

                if (_dist == 0)
                {
                    _centerAndZoom.ZoomLevel = _mapZoomLevels.DefaultZoom;
                }
                else
                {
                    // From Aiska Hendra
                    // http://www.codedblog.com/2007/09/28/how-to-auto-zoom-and-auto-center-google-maps/
                    var zoom = Math.Floor(8 - Math.Log(1.6446 * _dist / Math.Sqrt(2 * (_mapDisplay * _mapDisplay))) / Math.Log(2));
                    _centerAndZoom.ZoomLevel = Convert.ToInt32(zoom);
                }


                #region IfElse way of assigning values

                //if (miles == 0)
                //{
                //    _centerAndZoom.ZoomLevel = _mapZoomLevels.DefaultZoom;
                //}
                //else if (miles < .2)
                //{
                //    _centerAndZoom.ZoomLevel = _mapZoomLevels.MaxZoom;
                //}
                //else if (miles < 1)
                //    _centerAndZoom.ZoomLevel = 14;
                //else if (miles < 10)
                //    _centerAndZoom.ZoomLevel = 13;
                //else if (miles < 20)
                //    _centerAndZoom.ZoomLevel = 12;
                //else if (miles < 30)
                //    _centerAndZoom.ZoomLevel = 11;
                //else if (miles < 40)
                //    _centerAndZoom.ZoomLevel = 10;
                //else if (miles < 100)
                //    _centerAndZoom.ZoomLevel = 9;
                //else if (miles < 200)
                //    _centerAndZoom.ZoomLevel = 8;
                //else if (miles < 300)
                //    _centerAndZoom.ZoomLevel = 7;
                //else if (miles < 600)
                //    _centerAndZoom.ZoomLevel = 6;
                //else if (miles < 1000)
                //    _centerAndZoom.ZoomLevel = 5;
                //else if (miles < 2000)
                //    _centerAndZoom.ZoomLevel = 4;
                //else if (miles < 4000)
                //    _centerAndZoom.ZoomLevel = 3;
                //else if (miles < 6000)
                //    _centerAndZoom.ZoomLevel = 2;
                //else                    
                //    _centerAndZoom.ZoomLevel = 1;

                #endregion

            }

            return _centerAndZoom;
        }
        
        public static bool HasGoodCoordinates(GPSCoordinateModel coordinates)
        {
            if (coordinates == null || (!coordinates.Lat.HasValue) || (!coordinates.Lon.HasValue) || (coordinates.Lat == 0.0 && coordinates.Lon == 0.0))
            {
                return false;
            }

            return true;
        }

        public static int IsMultipleMarkersAtLocation(IList<MarkerModel> markers, Location location)
        {
            return markers.Where(m => m.Location.Latitude == location.Latitude && m.Location.Longitude == location.Longitude).Count();

        }

        public static Color GetRandomColor(IList<Color> usedColors)
        {
            if (usedColors == null)
                usedColors = new List<Color>();

            Random rdm = new Random();
            //Limit to 250 because I need a 3 point buffer for stroke. 255 would max me out
            int r = rdm.Next(0, 250);
            int b = rdm.Next(0, 250);
            int g = rdm.Next(0, 250);

            Color newColor = Color.FromArgb(255, (byte)r, (byte)g, (byte)b);

            if (usedColors.Contains(newColor))
                newColor = GetRandomColor(usedColors);

            usedColors.Add(newColor);

            return newColor;
        }

        // Method recommended by telerik to calculate center and zoom.
        // They state the radmapcontrol has to have a locationRect to use with
        // Internal settings pertaining to display to calculate this.
        public static LocationRect SetBestView(IEnumerable<MarkerModel> markers)
        {
            LocationRect bestView = new LocationRect();
            Location northEast = Location.Empty;
            Location southWest = Location.Empty;

            foreach (MarkerModel m in markers)
            {
                if (m.Coordinates != null && m.Coordinates.Lat.HasValue && m.Coordinates.Lon.HasValue)
                {
                    Location location = new Location(m.Coordinates.Lat.Value, m.Coordinates.Lon.Value);

                    if (northEast.IsEmpty)
                    {
                        northEast = location;
                    }
                    else
                    {
                        if (!location.IsEmpty)
                        {
                            northEast.Latitude = Math.Max(northEast.Latitude, location.Latitude);
                            northEast.Longitude = Math.Max(northEast.Longitude, location.Longitude);
                        }
                    }

                    if (southWest.IsEmpty)
                    {
                        southWest = location;
                    }
                    else
                    {
                        if (!location.IsEmpty)
                        {
                            southWest.Latitude = Math.Min(southWest.Latitude, location.Latitude);
                            southWest.Longitude = Math.Min(southWest.Longitude, location.Longitude);
                        }
                    }
                }
            }

            if (!northEast.IsEmpty && !southWest.IsEmpty)
            {
                bestView = new LocationRect(northEast, southWest);

                // This code would be to fire an event.
                //if (this.BestViewChanged != null) 
                //{ 
                //    BestViewEventArgs args = new BestViewEventArgs() 
                //    { 
                //        BestView = bestView  
                //    }; 
                //    this.BestViewChanged(this, args); 
                //} 
            }

            return bestView;
        } 

    }
}
