﻿using System;
using System.Linq;
using Northwind.Common.NwSvc;
using MVVM.Framework;

namespace Northwind.Common
{
    
    internal static class EmployeeModelAssembler<T> where T : EmployeeModel, new()
    {
        internal static SmartObservableCollection<T> AssembleContracts(Employee[] froms)
        {
            SmartObservableCollection<T> tos = new SmartObservableCollection<T>();

            foreach (Employee from in froms)
            {
                tos.Add(AssembleContract(from));
            }

            Array.Clear(froms, 0, froms.Length);

            return (SmartObservableCollection<T>)tos;
        }

        internal static T AssembleContract(Employee from)
        {
            T to = new T();

            to.EmployeeId.DataValue = from.EmployeeId;
            to.LastName.DataValue = from.LastName;
            to.FirstName.DataValue = from.FirstName;
            to.Title.DataValue = from.Title;
            to.TitleOfCourtesy.DataValue = from.TitleOfCourtesy;
            to.BirthDate.DataValue = from.BirthDate;
            to.HireDate.DataValue = from.HireDate;
            to.Address.DataValue = from.Address;
            to.City.DataValue = from.City;
            to.Region.DataValue = from.Region;
            to.PostalCode.DataValue = from.PostalCode;
            to.Country.DataValue = from.Country;
            to.HomePhone.DataValue = from.HomePhone;
            to.Extension.DataValue = from.Extension;

            if (from.Photo != null)
            {
                to.Photo.DataValue = from.Photo.ToArray();
            }

            to.Notes.DataValue = from.Notes;
            to.ReportsTo.DataValue = from.ReportsTo;
            to.FullName.DataValue = string.Format("{0}, {1}", from.LastName, from.FirstName);

            return to;
        }
    }
}
