﻿using System;
using System.Linq;
using Northwind.Common.NwSvc;
using MVVM.Framework;

namespace Northwind.Common
{
    internal static class OrderModelAssembler<T> where T : OrderModel, new()
    {
        internal static SmartObservableCollection<T> AssembleContracts(Order[] froms)
        {
            SmartObservableCollection<T> tos = new SmartObservableCollection<T>();

            foreach (Order from in froms)
            {
                tos.Add(AssembleContract(from));
            }

            return (SmartObservableCollection<T>)tos;
        }

        internal static T AssembleContract(Order from)
        {
            T to = new T();

            to.OrderId.DataValue = from.OrderId;
            to.CustomerId.DataValue = from.CustomerId;
            to.EmployeeId.DataValue = from.EmployeeId;
            to.OrderDate.DataValue = from.OrderDate;
            to.RequiredDate.DataValue = from.RequiredDate;
            to.ShippedDate.DataValue = from.ShippedDate;
            to.ShipVia.DataValue = from.ShipVia;
            to.Freight.DataValue = from.Freight;
            to.ShipName.DataValue = from.ShipName;
            to.ShipAddress.DataValue = from.ShipAddress;
            to.ShipCity.DataValue = from.ShipCity;
            to.ShipRegion.DataValue = from.ShipRegion;
            to.ShipPostalCode.DataValue = from.ShipPostalCode;
            to.ShipCountry.DataValue = from.ShipCountry;
            to.OrderStatusId.DataValue = from.OrderStatusId;
            to.Lat.DataValue = from.Lat;
            to.Lon.DataValue = from.Lon;

            if (from.Shipper != null)
            {
                to.Shipper = ShipperModelAssembler<ShipperModel>.AssembleContract(from.Shipper);
            }

            if (from.OrderStatus != null)
            {
                to.OrderStatus = OrderStatusModelAssembler<OrderStatusModel>.AssembleContract(from.OrderStatus);
            }

            if (from.OrderDetails != null)
            {
                to.OrderDetails.Repopulate(OrderDetailModelAssembler<OrderDetailModel>.AssembleContracts(from.OrderDetails));
            }

            if (from.Customer != null)
            {
                to.Customer = CustomerModelAssembler<CustomerModel>.AssembleContract(from.Customer);
            }

            if (from.Employee != null)
            {
                to.Employee = EmployeeModelAssembler<EmployeeModel>.AssembleContract(from.Employee);
            }

            return to;
        }
    }
}
