﻿using System;
using System.Linq;
using MVVM.Framework;
using System.ComponentModel;

namespace Northwind.Common
{
    public class CustomerModel : BaseEditableValidatingModel
    {

        #region Private Member Variables

        private DataWrapper<string> _customerId;
        private DataWrapper<string> _companyName;
        private DataWrapper<string> _contactName;
        private DataWrapper<string> _contactTitle;
        private DataWrapper<string> _address;
        private DataWrapper<string> _city;
        private DataWrapper<string> _region;
        private DataWrapper<string> _postalCode;
        private DataWrapper<string> _country;
        private DataWrapper<string> _phone;
        private DataWrapper<string> _fax;

        private bool _isSelected;


        #endregion

        #region ctor

        public CustomerModel()
        {
            CustomerId = new DataWrapper<string>(this, _customerIdChangeArgs);
            CompanyName = new DataWrapper<string>(this, _companyNameChangeArgs);
            ContactName = new DataWrapper<string>(this, _contactNameChangeArgs);
            ContactTitle = new DataWrapper<string>(this, _contactTitleChangeArgs);
            Address = new DataWrapper<string>(this, _addressChangeArgs);
            City = new DataWrapper<string>(this, _cityChangeArgs);
            Region = new DataWrapper<string>(this, _regionChangeArgs);
            PostalCode = new DataWrapper<string>(this, _postalCodeChangeArgs);
            Country = new DataWrapper<string>(this, _countryChangeArgs);
            Phone = new DataWrapper<string>(this, _phoneChangeArgs);
            Fax = new DataWrapper<string>(this, _faxChangeArgs);

            CachedListOfDataWrappers = DataWrapperHelper.GetWrapperProperties<CustomerModel>(this).ToList();
        }

        #endregion

        #region Public Properties

        static PropertyChangedEventArgs _customerIdChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.CustomerId);

        public DataWrapper<string> CustomerId
        {
            get { return _customerId; }
            private set
            {
                _customerId = value;
                NotifyPropertyChanged(_customerIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _companyNameChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.CompanyName);

        public DataWrapper<string> CompanyName
        {
            get { return _companyName; }
            private set
            {
                _companyName = value;
                NotifyPropertyChanged(_companyNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _contactNameChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.ContactName);

        public DataWrapper<string> ContactName
        {
            get { return _contactName; }
            private set
            {
                _contactName = value;
                NotifyPropertyChanged(_contactNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _contactTitleChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.ContactTitle);

        public DataWrapper<string> ContactTitle
        {
            get { return _contactTitle; }
            private set
            {
                _contactTitle = value;
                NotifyPropertyChanged(_contactTitleChangeArgs);
            }
        }

        static PropertyChangedEventArgs _addressChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.Address);

        public DataWrapper<string> Address
        {
            get { return _address; }
            private set
            {
                _address = value;
                NotifyPropertyChanged(_addressChangeArgs);
            }
        }

        static PropertyChangedEventArgs _cityChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.City);

        public DataWrapper<string> City
        {
            get { return _city; }
            private set
            {
                _city = value;
                NotifyPropertyChanged(_cityChangeArgs);
            }
        }

        static PropertyChangedEventArgs _regionChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.Region);

        public DataWrapper<string> Region
        {
            get { return _region; }
            private set
            {
                _region = value;
                NotifyPropertyChanged(_regionChangeArgs);
            }
        }

        static PropertyChangedEventArgs _postalCodeChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.PostalCode);

        public DataWrapper<string> PostalCode
        {
            get { return _postalCode; }
            private set
            {
                _postalCode = value;
                NotifyPropertyChanged(_postalCodeChangeArgs);
            }
        }

        static PropertyChangedEventArgs _countryChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.Country);

        public DataWrapper<string> Country
        {
            get { return _country; }
            private set
            {
                _country = value;
                NotifyPropertyChanged(_countryChangeArgs);
            }
        }

        static PropertyChangedEventArgs _phoneChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.Phone);

        public DataWrapper<string> Phone
        {
            get { return _phone; }
            private set
            {
                _phone = value;
                NotifyPropertyChanged(_phoneChangeArgs);
            }
        }

        static PropertyChangedEventArgs _faxChangeArgs =
        ObservableHelper.CreateArgs<CustomerModel>(x => x.Fax);

        public DataWrapper<string> Fax
        {
            get { return _fax; }
            private set
            {
                _fax = value;
                NotifyPropertyChanged(_faxChangeArgs);
            }
        }

        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (_isSelected != value)
                {
                    _isSelected = value;
                    NotifyPropertyChanged(ReflectionUtility.GetPropertyName(() => IsSelected));
                }
            }
        }

        #endregion



    }
}
