﻿using System;
using System.Linq;
using MVVM.Framework;
using System.ComponentModel;


namespace Northwind.Common
{
    public class EmployeeModel : BaseEditableValidatingModel
    {
        #region ctor

        public EmployeeModel()
        {
            EmployeeId = new DataWrapper<int?>(this, _employeeIdChangeArgs);
            LastName = new DataWrapper<string>(this, _lastNameChangeArgs);
            FirstName = new DataWrapper<string>(this, _firstNameChangeArgs);
            Title = new DataWrapper<string>(this, _titleChangeArgs);
            TitleOfCourtesy = new DataWrapper<string>(this, _titleOfCourtesyChangeArgs);
            BirthDate = new DataWrapper<DateTime?>(this, _birthDateChangeArgs);
            HireDate = new DataWrapper<DateTime?>(this, _hireDateChangeArgs);
            Address = new DataWrapper<string>(this, _addressChangeArgs);
            City = new DataWrapper<string>(this, _cityChangeArgs);
            Region = new DataWrapper<string>(this, _regionChangeArgs);
            PostalCode = new DataWrapper<string>(this, _postalCodeChangeArgs);
            Country = new DataWrapper<string>(this, _countryChangeArgs);
            HomePhone = new DataWrapper<string>(this, _homePhoneChangeArgs);
            Extension = new DataWrapper<string>(this, _extensionChangeArgs);
            Photo = new DataWrapper<byte[]>(this, _photoChangeArgs);
            Notes = new DataWrapper<string>(this, _notesChangeArgs);
            ReportsTo = new DataWrapper<int?>(this, _reportsToChangeArgs);
            PhotoPath = new DataWrapper<string>(this, _photoPathChangeArgs);
            Manager = new DataWrapper<EmployeeModel>(this, _managerChangeArgs);
            FullName = new DataWrapper<string>(this, _fullNameChangeArgs);

            CachedListOfDataWrappers = DataWrapperHelper.GetWrapperProperties<EmployeeModel>(this).ToList();
        }

        #endregion


        #region Private Member Variables

        private DataWrapper<int?> _employeeId;
        private DataWrapper<string> _lastName;
        private DataWrapper<string> _firstName;
        private DataWrapper<string> _title;
        private DataWrapper<string> _titleOfCourtesy;
        private DataWrapper<DateTime?> _birthDate;
        private DataWrapper<DateTime?> _hireDate;
        private DataWrapper<string> _address;
        private DataWrapper<string> _city;
        private DataWrapper<string> _region;
        private DataWrapper<string> _postalCode;
        private DataWrapper<string> _country;
        private DataWrapper<string> _homePhone;
        private DataWrapper<string> _extension;
        private DataWrapper<byte[]> _photo;
        private DataWrapper<string> _notes;
        private DataWrapper<int?> _reportsTo;
        private DataWrapper<string> _photoPath;
        private DataWrapper<EmployeeModel> _manager;
        private DataWrapper<string> _fullName;

        #endregion

        #region Public Properties

        static PropertyChangedEventArgs _employeeIdChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.EmployeeId);

        public DataWrapper<int?> EmployeeId
        {
            get { return _employeeId; }
            private set
            {
                _employeeId = value;
                NotifyPropertyChanged(_employeeIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _lastNameChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.LastName);

        public DataWrapper<string> LastName
        {
            get { return _lastName; }
            private set
            {
                _lastName = value;
                NotifyPropertyChanged(_lastNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _firstNameChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.FirstName);

        public DataWrapper<string> FirstName
        {
            get { return _firstName; }
            private set
            {
                _firstName = value;
                NotifyPropertyChanged(_firstNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _titleChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Title);

        public DataWrapper<string> Title
        {
            get { return _title; }
            private set
            {
                _title = value;
                NotifyPropertyChanged(_titleChangeArgs);
            }
        }

        static PropertyChangedEventArgs _titleOfCourtesyChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.TitleOfCourtesy);

        public DataWrapper<string> TitleOfCourtesy
        {
            get { return _titleOfCourtesy; }
            private set
            {
                _titleOfCourtesy = value;
                NotifyPropertyChanged(_titleOfCourtesyChangeArgs);
            }
        }

        static PropertyChangedEventArgs _birthDateChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.BirthDate);

        public DataWrapper<DateTime?> BirthDate
        {
            get { return _birthDate; }
            private set
            {
                _birthDate = value;
                NotifyPropertyChanged(_birthDateChangeArgs);
            }
        }

        static PropertyChangedEventArgs _hireDateChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.HireDate);

        public DataWrapper<DateTime?> HireDate
        {
            get { return _hireDate; }
            private set
            {
                _hireDate = value;
                NotifyPropertyChanged(_hireDateChangeArgs);
            }
        }

        static PropertyChangedEventArgs _addressChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Address);

        public DataWrapper<string> Address
        {
            get { return _address; }
            private set
            {
                _address = value;
                NotifyPropertyChanged(_addressChangeArgs);
            }
        }

        static PropertyChangedEventArgs _cityChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.City);

        public DataWrapper<string> City
        {
            get { return _city; }
            private set
            {
                _city = value;
                NotifyPropertyChanged(_cityChangeArgs);
            }
        }

        static PropertyChangedEventArgs _regionChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Region);

        public DataWrapper<string> Region
        {
            get { return _region; }
            private set
            {
                _region = value;
                NotifyPropertyChanged(_regionChangeArgs);
            }
        }

        static PropertyChangedEventArgs _postalCodeChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.PostalCode);

        public DataWrapper<string> PostalCode
        {
            get { return _postalCode; }
            private set
            {
                _postalCode = value;
                NotifyPropertyChanged(_postalCodeChangeArgs);
            }
        }

        static PropertyChangedEventArgs _countryChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Country);

        public DataWrapper<string> Country
        {
            get { return _country; }
            private set
            {
                _country = value;
                NotifyPropertyChanged(_countryChangeArgs);
            }
        }

        static PropertyChangedEventArgs _homePhoneChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.HomePhone);

        public DataWrapper<string> HomePhone
        {
            get { return _homePhone; }
            private set
            {
                _homePhone = value;
                NotifyPropertyChanged(_homePhoneChangeArgs);
            }
        }

        static PropertyChangedEventArgs _extensionChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Extension);

        public DataWrapper<string> Extension
        {
            get { return _extension; }
            private set
            {
                _extension = value;
                NotifyPropertyChanged(_extensionChangeArgs);
            }
        }

        static PropertyChangedEventArgs _photoChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Photo);

        public DataWrapper<byte[]> Photo
        {
            get { return _photo; }
            private set
            {
                _photo = value;
                NotifyPropertyChanged(_photoChangeArgs);
            }
        }

        static PropertyChangedEventArgs _notesChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Notes);

        public DataWrapper<string> Notes
        {
            get { return _notes; }
            private set
            {
                _notes = value;
                NotifyPropertyChanged(_notesChangeArgs);
            }
        }

        static PropertyChangedEventArgs _reportsToChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.ReportsTo);

        public DataWrapper<int?> ReportsTo
        {
            get { return _reportsTo; }
            private set
            {
                _reportsTo = value;
                NotifyPropertyChanged(_reportsToChangeArgs);
            }
        }

        static PropertyChangedEventArgs _photoPathChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.PhotoPath);

        public DataWrapper<string> PhotoPath
        {
            get { return _photoPath; }
            private set
            {
                _photoPath = value;
                NotifyPropertyChanged(_photoPathChangeArgs);
            }
        }

        static PropertyChangedEventArgs _managerChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.Manager);

        public DataWrapper<EmployeeModel> Manager
        {
            get { return _manager; }
            private set
            {
                _manager = value;
                NotifyPropertyChanged(_managerChangeArgs);
            }
        }

        static PropertyChangedEventArgs _fullNameChangeArgs =
        ObservableHelper.CreateArgs<EmployeeModel>(x => x.FullName);

        public DataWrapper<string> FullName
        {
            get { return _fullName; }
            private set
            {
                _fullName = value;
                NotifyPropertyChanged(_fullNameChangeArgs);
            }
        }

        #endregion


       
    }
}
