﻿using System;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Controls.Map;
using System.Collections.Generic;
using MVVM.Framework;
using System.Windows;

namespace Northwind.Common
{
    public class MarkerModel : INPCBase
    {
        private List<Color> _usedColors = new List<Color>();
        private Visibility _toolTipVisibility;

        public MarkerModel(OrderModel orderModel, Enums.MarkerType markerType)
        {
            Color rdmColor = MapHelpers.GetRandomColor(_usedColors);
            Color rdmStrokeColor = Color.FromArgb((byte)100, (byte)(rdmColor.R + 3), (byte)(rdmColor.G + 3), (byte)(rdmColor.B + 3));

            GPSCoordinateModel _gpsCoord = new GPSCoordinateModel();
            _gpsCoord.Lat = orderModel.Lat.DataValue;
            _gpsCoord.Lon = orderModel.Lon.DataValue;

            Coordinates = _gpsCoord;

            Index = orderModel.Index;
            //Index = orderModel.OrderId.DataValue.Value;
            //LabelData = "OrderId: " + orderModel.OrderId.DataValue;
            LabelData = orderModel.Index.ToString();
            Type = markerType;
            Fill = rdmColor;
            Stroke = rdmStrokeColor;

            if (markerType == Enums.MarkerType.MultiPoint)
            {
                ToolTipVisibility = Visibility.Collapsed;
            }

        }

        public GPSCoordinateModel Coordinates { get; set; }
        public int Index { get; set; }
        public Northwind.Common.Enums.MarkerType Type { get; set; }
        public Color Fill { get; set; }
        public Color Stroke { get; set; }
        public string LabelData { get; set; }

        /// <summary>
        /// Gets coordinates of the marker as the RadMap 
        /// Location structure. It is used to bind
        /// MapLayer.Location attachable property to.
        /// </summary>
        public Location Location
        {
            get
            {
                Location location = new Location(
                    this.Coordinates.Lat.Value,
                    this.Coordinates.Lon.Value);

                return location;
            }
        }

        // Controls whether the ToolTip is visible or not.
        public Visibility ToolTipVisibility
        {
            get { return _toolTipVisibility; }
            set
            {
                if (_toolTipVisibility != value)
                {
                    _toolTipVisibility = value;
                    RaisePropertyChanged("ToolTipVisibility");
                }
            }
        }
    }
}
