﻿using System;
using System.Linq;
using MVVM.Framework;
using System.ComponentModel;

namespace Northwind.Common
{
    public class OrderDetailModel : BaseEditableValidatingModel
    {
        #region ctor

        public OrderDetailModel()
        {
            OrderId = new DataWrapper<int>(this, _orderIdChangeArgs);
            ProductId = new DataWrapper<int>(this, _productIdChangeArgs);
            UnitPrice = new DataWrapper<decimal>(this, _unitPriceChangeArgs);
            Quantity = new DataWrapper<short>(this, _quantityChangeArgs);
            Discount = new DataWrapper<float>(this, _discountChangeArgs);

            CachedListOfDataWrappers = DataWrapperHelper.GetWrapperProperties<OrderDetailModel>(this).ToList();
        }

	    #endregion


        #region Private Member Variables

        private DataWrapper<int> _orderId;
        private DataWrapper<int> _productId;
        private DataWrapper<decimal> _unitPrice;
        private DataWrapper<short> _quantity;
        private DataWrapper<float> _discount;
        private DataWrapper<ProductModel> _product;

        #endregion

        #region Public Properties

        static PropertyChangedEventArgs _orderIdChangeArgs =
        ObservableHelper.CreateArgs<OrderDetailModel>(x => x.OrderId);

        public DataWrapper<int> OrderId
        {
            get { return _orderId; }
            private set
            {
                _orderId = value;
                NotifyPropertyChanged(_orderIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _productIdChangeArgs =
        ObservableHelper.CreateArgs<OrderDetailModel>(x => x.ProductId);

        public DataWrapper<int> ProductId
        {
            get { return _productId; }
            private set
            {
                _productId = value;
                NotifyPropertyChanged(_productIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _unitPriceChangeArgs =
        ObservableHelper.CreateArgs<OrderDetailModel>(x => x.UnitPrice);

        public DataWrapper<decimal> UnitPrice
        {
            get { return _unitPrice; }
            private set
            {
                _unitPrice = value;
                NotifyPropertyChanged(_unitPriceChangeArgs);
            }
        }

        static PropertyChangedEventArgs _quantityChangeArgs =
        ObservableHelper.CreateArgs<OrderDetailModel>(x => x.Quantity);

        public DataWrapper<short> Quantity
        {
            get { return _quantity; }
            private set
            {
                _quantity = value;
                NotifyPropertyChanged(_quantityChangeArgs);
            }
        }

        static PropertyChangedEventArgs _discountChangeArgs =
        ObservableHelper.CreateArgs<OrderDetailModel>(x => x.Discount);

        public DataWrapper<float> Discount
        {
            get { return _discount; }
            private set
            {
                _discount = value;
                NotifyPropertyChanged(_discountChangeArgs);
            }
        }

        public ProductModel Product { get; set; }

        #endregion


       
    }
}
