﻿using System;
using System.Linq;
using MVVM.Framework;
using System.Collections.Generic;
using System.ComponentModel;

namespace Northwind.Common
{
    public class OrderModel : BaseEditableValidatingModel
    {

        #region ctor

        public OrderModel()
        {
            OrderId = new DataWrapper<int?>(this, _orderIdChangeArgs);
            CustomerId = new DataWrapper<string>(this, _customerIdChangeArgs);
            EmployeeId = new DataWrapper<int?>(this, _employeeIdChangeArgs);
            OrderDate = new DataWrapper<DateTime?>(this, _orderDateChangeArgs);
            RequiredDate = new DataWrapper<DateTime?>(this, _requiredDateChangeArgs);
            ShippedDate = new DataWrapper<DateTime?>(this, _shippedDateChangeArgs);
            Freight = new DataWrapper<decimal?>(this, _freightChangeArgs);
            ShipVia = new DataWrapper<int?>(this, _shipViaChangeArgs);
            ShipName = new DataWrapper<string>(this, _shipNameChangeArgs);
            ShipAddress = new DataWrapper<string>(this, _shipAddressChangeArgs);
            ShipCity = new DataWrapper<string>(this, _shipCityChangeArgs);
            ShipRegion = new DataWrapper<string>(this, _shipRegionChangeArgs);
            ShipPostalCode = new DataWrapper<string>(this, _shipPostalCodeChangeArgs);
            ShipCountry = new DataWrapper<string>(this, _shipCountryChangeArgs);
            OrderStatusId = new DataWrapper<short>(this, _orderStatusIdChangeArgs);
            Lat = new DataWrapper<float?>(this, _latChangeArgs);
            Lon = new DataWrapper<float?>(this, _lonChangeArgs);

            OrderDetails = new SmartObservableCollection<OrderDetailModel>();

            CachedListOfDataWrappers = DataWrapperHelper.GetWrapperProperties<OrderModel>(this).ToList();
        }

        #endregion
        
        #region Private Member Variables

        private DataWrapper<int?> _orderId;
        private DataWrapper<string> _customerId;
        private DataWrapper<int?> _employeeId;
        private DataWrapper<DateTime?> _orderDate;
        private DataWrapper<DateTime?> _requiredDate;
        private DataWrapper<DateTime?> _shippedDate;
        private DataWrapper<decimal?> _freight;
        private DataWrapper<int?> _shipVia;
        private DataWrapper<string> _shipName;
        private DataWrapper<string> _shipAddress;
        private DataWrapper<string> _shipCity;
        private DataWrapper<string> _shipRegion;
        private DataWrapper<string> _shipPostalCode;
        private DataWrapper<string> _shipCountry;
        private DataWrapper<short> _orderStatusId;
        private DataWrapper<float?> _lat;
        private DataWrapper<float?> _lon;


        #region Public Properties


        #endregion


        #endregion

        #region Public Properties

        static PropertyChangedEventArgs _orderIdChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.OrderId);

        public DataWrapper<int?> OrderId
        {
            get { return _orderId; }
            private set
            {
                _orderId = value;
                NotifyPropertyChanged(_orderIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _customerIdChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.CustomerId);

        public DataWrapper<string> CustomerId
        {
            get { return _customerId; }
            private set
            {
                _customerId = value;
                NotifyPropertyChanged(_customerIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _employeeIdChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.EmployeeId);

        public DataWrapper<int?> EmployeeId
        {
            get { return _employeeId; }
            private set
            {
                _employeeId = value;
                NotifyPropertyChanged(_employeeIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _orderDateChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.OrderDate);

        public DataWrapper<DateTime?> OrderDate
        {
            get { return _orderDate; }
            private set
            {
                _orderDate = value;
                NotifyPropertyChanged(_orderDateChangeArgs);
            }
        }

        static PropertyChangedEventArgs _requiredDateChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.RequiredDate);

        public DataWrapper<DateTime?> RequiredDate
        {
            get { return _requiredDate; }
            private set
            {
                _requiredDate = value;
                NotifyPropertyChanged(_requiredDateChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shippedDateChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShippedDate);

        public DataWrapper<DateTime?> ShippedDate
        {
            get { return _shippedDate; }
            private set
            {
                _shippedDate = value;
                NotifyPropertyChanged(_shippedDateChangeArgs);
            }
        }

        static PropertyChangedEventArgs _freightChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.Freight);

        public DataWrapper<decimal?> Freight
        {
            get { return _freight; }
            private set
            {
                _freight = value;
                NotifyPropertyChanged(_freightChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipViaChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipVia);

        public DataWrapper<int?> ShipVia
        {
            get { return _shipVia; }
            private set
            {
                _shipVia = value;
                NotifyPropertyChanged(_shipViaChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipNameChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipName);

        public DataWrapper<string> ShipName
        {
            get { return _shipName; }
            private set
            {
                _shipName = value;
                NotifyPropertyChanged(_shipNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipAddressChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipAddress);

        public DataWrapper<string> ShipAddress
        {
            get { return _shipAddress; }
            private set
            {
                _shipAddress = value;
                NotifyPropertyChanged(_shipAddressChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipCityChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipCity);

        public DataWrapper<string> ShipCity
        {
            get { return _shipCity; }
            private set
            {
                _shipCity = value;
                NotifyPropertyChanged(_shipCityChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipRegionChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipRegion);

        public DataWrapper<string> ShipRegion
        {
            get { return _shipRegion; }
            private set
            {
                _shipRegion = value;
                NotifyPropertyChanged(_shipRegionChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipPostalCodeChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipPostalCode);

        public DataWrapper<string> ShipPostalCode
        {
            get { return _shipPostalCode; }
            private set
            {
                _shipPostalCode = value;
                NotifyPropertyChanged(_shipPostalCodeChangeArgs);
            }
        }

        static PropertyChangedEventArgs _shipCountryChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.ShipCountry);

        public DataWrapper<string> ShipCountry
        {
            get { return _shipCountry; }
            private set
            {
                _shipCountry = value;
                NotifyPropertyChanged(_shipCountryChangeArgs);
            }
        }

        static PropertyChangedEventArgs _orderStatusIdChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.OrderStatusId);

        public DataWrapper<short> OrderStatusId
        {
            get { return _orderStatusId; }
            private set
            {
                _orderStatusId = value;
                NotifyPropertyChanged(_orderStatusIdChangeArgs);
            }
        }


        static PropertyChangedEventArgs _latChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.Lat);

        public DataWrapper<float?> Lat
        {
            get { return _lat; }
            private set
            {
                _lat = value;
                NotifyPropertyChanged(_latChangeArgs);
            }
        }

        static PropertyChangedEventArgs _lonChangeArgs =
        ObservableHelper.CreateArgs<OrderModel>(x => x.Lon);

        public DataWrapper<float?> Lon
        {
            get { return _lon; }
            private set
            {
                _lon = value;
                NotifyPropertyChanged(_lonChangeArgs);
            }
        }

        #endregion

        // Used to distinguish orders on a map.
        public int Index { get; set; }
        public ShipperModel Shipper { get; set; }
        public OrderStatusModel OrderStatus { get; set; }
        public EmployeeModel Employee { get; set; }
        public CustomerModel Customer { get; set; }
        public SmartObservableCollection<OrderDetailModel> OrderDetails { get; private set; }

        private bool _isSelected;

        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (_isSelected != value)
                {
                    _isSelected = value;
                    NotifyPropertyChanged(ReflectionUtility.GetPropertyName(() => IsSelected));
                }
            }
        }


    }
}
