﻿using System;
using System.Linq;
using MVVM.Framework;
using System.ComponentModel;

namespace Northwind.Common
{
    public class ProductModel : BaseEditableValidatingModel
    {

        #region ctor

        public ProductModel()
        {
            ProductId = new DataWrapper<int?>(this, _productIdChangeArgs);
            ProductName = new DataWrapper<string>(this, _productNameChangeArgs);
            SupplierId = new DataWrapper<int?>(this, _supplierIdChangeArgs);
            CategoryId = new DataWrapper<int?>(this, _categoryIdChangeArgs);
            QuantityPerUnit = new DataWrapper<string>(this, _quantityPerUnitChangeArgs);
            UnitPrice = new DataWrapper<decimal?>(this, _unitPriceChangeArgs);
            UnitsInStock = new DataWrapper<short?>(this, _unitsInStockChangeArgs);
            UnitsOnOrder = new DataWrapper<short?>(this, _unitsOnOrderChangeArgs);
            ReorderLevel = new DataWrapper<short?>(this, _reorderLevelChangeArgs);
            Discontinued = new DataWrapper<bool>(this, _discontinuedChangeArgs);

            CachedListOfDataWrappers = DataWrapperHelper.GetWrapperProperties<ProductModel>(this).ToList();
        }

        #endregion


        #region Private Member Variables

        private DataWrapper<int?> _productId;
        private DataWrapper<string> _productName;
        private DataWrapper<int?> _supplierId;
        private DataWrapper<int?> _categoryId;
        private DataWrapper<string> _quantityPerUnit;
        private DataWrapper<decimal?> _unitPrice;
        private DataWrapper<short?> _unitsInStock;
        private DataWrapper<short?> _unitsOnOrder;
        private DataWrapper<short?> _reorderLevel;
        private DataWrapper<bool> _discontinued;

        #endregion

        #region Public Properties

        static PropertyChangedEventArgs _productIdChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.ProductId);

        public DataWrapper<int?> ProductId
        {
            get { return _productId; }
            private set
            {
                _productId = value;
                NotifyPropertyChanged(_productIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _productNameChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.ProductName);

        public DataWrapper<string> ProductName
        {
            get { return _productName; }
            private set
            {
                _productName = value;
                NotifyPropertyChanged(_productNameChangeArgs);
            }
        }

        static PropertyChangedEventArgs _supplierIdChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.SupplierId);

        public DataWrapper<int?> SupplierId
        {
            get { return _supplierId; }
            private set
            {
                _supplierId = value;
                NotifyPropertyChanged(_supplierIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _categoryIdChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.CategoryId);

        public DataWrapper<int?> CategoryId
        {
            get { return _categoryId; }
            private set
            {
                _categoryId = value;
                NotifyPropertyChanged(_categoryIdChangeArgs);
            }
        }

        static PropertyChangedEventArgs _quantityPerUnitChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.QuantityPerUnit);

        public DataWrapper<string> QuantityPerUnit
        {
            get { return _quantityPerUnit; }
            private set
            {
                _quantityPerUnit = value;
                NotifyPropertyChanged(_quantityPerUnitChangeArgs);
            }
        }

        static PropertyChangedEventArgs _unitPriceChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.UnitPrice);

        public DataWrapper<decimal?> UnitPrice
        {
            get { return _unitPrice; }
            private set
            {
                _unitPrice = value;
                NotifyPropertyChanged(_unitPriceChangeArgs);
            }
        }

        static PropertyChangedEventArgs _unitsInStockChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.UnitsInStock);

        public DataWrapper<short?> UnitsInStock
        {
            get { return _unitsInStock; }
            private set
            {
                _unitsInStock = value;
                NotifyPropertyChanged(_unitsInStockChangeArgs);
            }
        }

        static PropertyChangedEventArgs _unitsOnOrderChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.UnitsOnOrder);

        public DataWrapper<short?> UnitsOnOrder
        {
            get { return _unitsOnOrder; }
            private set
            {
                _unitsOnOrder = value;
                NotifyPropertyChanged(_unitsOnOrderChangeArgs);
            }
        }

        static PropertyChangedEventArgs _reorderLevelChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.ReorderLevel);

        public DataWrapper<short?> ReorderLevel
        {
            get { return _reorderLevel; }
            private set
            {
                _reorderLevel = value;
                NotifyPropertyChanged(_reorderLevelChangeArgs);
            }
        }

        static PropertyChangedEventArgs _discontinuedChangeArgs =
        ObservableHelper.CreateArgs<ProductModel>(x => x.Discontinued);

        public DataWrapper<bool> Discontinued
        {
            get { return _discontinued; }
            private set
            {
                _discontinued = value;
                NotifyPropertyChanged(_discontinuedChangeArgs);
            }
        }

        #endregion


        

    }
}
