﻿using System;
using System.Linq;
using System.ComponentModel.Composition;
using System.Collections.Generic;
using MVVM.Framework;
using Northwind.Common;
using System.Net;
using System.Net.Browser;
using Northwind.Common.NwSvc;


namespace Northwind.Common
{
    public interface INWClientService
    {
        void GetCustomersAsync(GetCustomersRequest request);
        event EventHandler<ServiceResultArgs<CustomerModel>> GetCustomersComplete;

        void GetOrdersAsync(GetOrdersRequest request);
        event EventHandler<ServiceResultArgs<OrderModel>> GetOrdersComplete;

    }

    [Export(typeof(INWClientService))]
    [PartCreationPolicy(System.ComponentModel.Composition.CreationPolicy.NonShared)]
    public class NWClientService : INWClientService
    {
        NorthwindServiceClient _nwService;

        #region ctor

        public NWClientService()
        {
            WebRequest.RegisterPrefix("http://", WebRequestCreator.ClientHttp);
            WebRequest.RegisterPrefix("https://", WebRequestCreator.ClientHttp);

            _nwService = new NorthwindServiceClient("NSV.NW.ServiceImplementation.NorthwindService", GlobalValues.NWSvcURI);

            _nwService.GetCustomersCompleted += new EventHandler<GetCustomersCompletedEventArgs>(NWService_GetCustomersCompleted);
            _nwService.GetOrdersCompleted += new EventHandler<GetOrdersCompletedEventArgs>(NWService_GetOrdersCompleted);
        }

        void NWService_GetCustomersCompleted(object sender, GetCustomersCompletedEventArgs e)
        {
            if (GetCustomersComplete != null)
            {
                if (e.Error == null)
                {
                    SmartObservableCollection<CustomerModel> _objs;

                    if (e.Result == null || e.Result.Customers == null)
                    {
                        _objs = new SmartObservableCollection<CustomerModel>();
                    }
                    else
                    {
                        _objs = CustomerModelAssembler<CustomerModel>.AssembleContracts(e.Result.Customers);
                    }

                    GetCustomersComplete(this, new ServiceResultArgs<CustomerModel>(_objs));

                    _objs = null;
                }
                else
                {
                    GetCustomersComplete(this, new ServiceResultArgs<CustomerModel>(e.Error));
                }
            }
        }

        public void GetCustomersAsync(GetCustomersRequest request)
        {
            _nwService.GetCustomersAsync(request);
        }

        public void GetOrdersAsync(GetOrdersRequest request)
        {
            _nwService.GetOrdersAsync(request);
        }

        void NWService_GetOrdersCompleted(object sender, GetOrdersCompletedEventArgs e)
        {
            if (GetOrdersComplete != null)
            {
                if (e.Error == null)
                {
                    SmartObservableCollection<OrderModel> _objs;

                    if (e.Result == null || e.Result.Orders == null)
                    {
                        _objs = new SmartObservableCollection<OrderModel>();
                    }
                    else
                    {
                        _objs = OrderModelAssembler<OrderModel>.AssembleContracts(e.Result.Orders);
                    }

                    GetOrdersComplete(this, new ServiceResultArgs<OrderModel>(_objs));

                    _objs = null;
                }
                else
                {
                    GetOrdersComplete(this, new ServiceResultArgs<OrderModel>(e.Error));
                }
            }
        }


        #endregion

        #region INorthwindClientService Members

        public event EventHandler<ServiceResultArgs<CustomerModel>> GetCustomersComplete;
        public event EventHandler<ServiceResultArgs<OrderModel>> GetOrdersComplete;

        #endregion
    }
}
