﻿using System;
using System.ComponentModel.Composition;
using Microsoft.Practices.Prism.Regions;
using Northwind.Common;
using MVVM.Framework;
using System.Collections.Generic;
using Telerik.Windows.Controls;
using System.Windows.Input;
using System.Windows;

namespace Northwind.Modules.Orders
{
    [Export(typeof(IOrderController))]
    public class OrderController : IOrderController
    {
        IRegionManager _regionManager;
        IRegionManager _orderRegionManager;

        OrderRootTemplateView _orderRootTemplateView;
        OrderView _orderView;
        RootToolBarView _rootToolBarView;
        RootRibbonBarTabView _rootRibbonBarTabView;
        
        IPopupController _popupController;

        [ImportingConstructor]
        public OrderController(IRegionManager regionManager, OrderRootTemplateView orderRootTemplateView, OrderView orderView,
            RootToolBarView rootToolBarView, RootRibbonBarTabView rootRibbonBarTabView)
        {
            try
            {
                _popupController = new PopupController(regionManager);

                //MapOrderView mapOrderView,
                //IPopupController popupController
                _regionManager = regionManager;
                _orderRootTemplateView = orderRootTemplateView;

                _orderView = orderView;
                _orderView.ViewModel.OrderController = this;

                //_rootToolBarView = rootToolBarView;
                _rootToolBarView = rootToolBarView;
                //_rootToolBarView.ViewModel.PopulateQATBButtons();
                _rootRibbonBarTabView = rootRibbonBarTabView;
                _rootRibbonBarTabView.ViewModel.OrderController = this;

                RegisterViewToRegions();
            }
            catch (Exception ex)
            {
                GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(ex, "Exception loading NorthwindController: "));
            }
        }

        #region Public Methods

        public void CloseMapOrderView()
        {
            _popupController.CloseMainPopup(true);
        }

        public void ShowMapOrderView(SmartObservableCollection<OrderModel> orders)
        {
            MapOrderView _mapOrderView = new MapOrderView();
            _mapOrderView.ViewModel.OrderController = this;

            if (orders == null || orders.Count < 1)
                MessageBox.Show("ShowMapOrderView must contain orders.");
            else
            {
                _mapOrderView.ViewModel.Initialize(orders); ;

                _popupController.ShowMainPopup(_mapOrderView);

            }
        }

        #endregion

        #region Private Methods

        private void RegisterViewToRegions()
        {            
            // Get the RootContentRegion defined in the main Silverlight assembly
            IRegion _rootContentRegion = _regionManager.Regions[GlobalRegionNames.RootContentRegion];
            // Add this modules template to it.
            _rootContentRegion.Add(_orderRootTemplateView, ReflectionUtility.GetPropertyName(() => _orderRootTemplateView));

            // We want to create get the region we added above to from the global app RegionManager
            IRegion _northwindRootTemplateRegion = _regionManager.Regions[OrderRegionNames.NorthwindRootTemplateRegion];
            // Now we want to create a regionManager for this module which will be used to manage regions specific to this module.
            _orderRegionManager = _northwindRootTemplateRegion.Add(_orderView, ReflectionUtility.GetPropertyName(() => _northwindRootTemplateRegion));

            // Now we need to get the QATB region from the main RibbonBar.This will place QATB in the root Quick Access Toolbar
            IRegion _rootToolBarRegion = _regionManager.Regions[GlobalRegionNames.RootToolbarRegion];
            _rootToolBarRegion.Add(_rootToolBarView, ReflectionUtility.GetPropertyName(() => _rootToolBarView));

            IRegion _rootRibbonTabRegion = _regionManager.Regions[GlobalRegionNames.RootRibbonTabRegion];
            _rootRibbonTabRegion.Add(_rootRibbonBarTabView, ReflectionUtility.GetPropertyName(() => _rootRibbonBarTabView));
        }

        #endregion

        #region Public Properties


        #endregion
    }
}
