﻿using System;
using System.Linq;
using Northwind.Common;
using MVVM.Framework;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Messaging;
using System.ComponentModel.Composition;
using GalaSoft.MvvmLight.Command;
using Telerik.Windows.Controls.Map;

namespace Northwind.Modules.Orders
{
    [Export]
    public class MapOrderViewModel : ViewModelBase, ICleanup
    {
        const int _defaultZoomLevel = 7;

        private Location _mapCenter;
        private SmartObservableCollection<MarkerModel> _orderMarkers;
        private SmartObservableCollection<MarkerModel> _orderMarkersMulti;

        private SmartObservableCollection<OrderModel> _orders;
        private int _zoomLevel;

        private RelayCommand _closeMapOrderViewCommand;

        public RelayCommand CloseMapOrderViewCommand
        {
            get
            {
                return _closeMapOrderViewCommand ?? (_closeMapOrderViewCommand = new RelayCommand(() =>
                    {
                        OrderController.CloseMapOrderView();
                    }
                    ));
            }
        }
        private DistanceUnit _distanceUnitMeasure;

        private RelayCommand<OrderModel> _mapOrderCommand;
        
        #region Constructor

        public MapOrderViewModel()
        {
            Orders = new SmartObservableCollection<OrderModel>();
            OrderMarkers = new SmartObservableCollection<MarkerModel>();
            OrderMarkersMulti = new SmartObservableCollection<MarkerModel>();
            
            // Setting the Distance for the map to miles.
            DistanceUnitMeasure = DistanceUnit.Mile;

            //Orders[0].ShipName.DataValue
        }

        #endregion

        #region Public Properties

        public DistanceUnit DistanceUnitMeasure
        {
            get { return _distanceUnitMeasure; }
            set
            {
                if (_distanceUnitMeasure != value)
                {
                    _distanceUnitMeasure = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => DistanceUnitMeasure));
                }
            }
        }

        public Location MapCenter
        {
            get { return _mapCenter; }
            set
            {
                if (_mapCenter != value)
                {
                    _mapCenter = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => MapCenter));
                }
            }
        }

        public IOrderController OrderController { get; set; }

        public SmartObservableCollection<MarkerModel> OrderMarkers
        {
            get { return _orderMarkers; }
            set
            {
                if (_orderMarkers != value)
                {
                    _orderMarkers = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => OrderMarkers));
                }
            }
        }

        public SmartObservableCollection<MarkerModel> OrderMarkersMulti
        {
            get { return _orderMarkersMulti; }
            set
            {
                if (_orderMarkersMulti != value)
                {
                    _orderMarkersMulti = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => OrderMarkersMulti));
                }
            }
        }

        public SmartObservableCollection<OrderModel> Orders
        {
            get { return _orders; }
            private set
            {
                if (_orders != value)
                {
                    _orders = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => Orders));
                }
            }
        }

        public int ZoomLevel
        {
            get { return _zoomLevel; }
            set
            {
                if (_zoomLevel != value)
                {
                    _zoomLevel = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => ZoomLevel));
                }
            }
        }

        public RelayCommand<OrderModel> MapOrderCommand
        {
            get
            {
                return _mapOrderCommand ?? (_mapOrderCommand = new RelayCommand<OrderModel>((a) =>
                    {
                        // If OrderId is null, then user selected to map all
                        if (!a.OrderId.DataValue.HasValue)
                        {
                            foreach (var o in Orders)
                            {
                                o.IsSelected = a.IsSelected;
                            }
                        }

                        OrderMarkers.Clear();
                        OrderMarkersMulti.Clear();

                        foreach(var o in Orders.Where(o => o.IsSelected && o.OrderId.DataValue.HasValue))
                        {
                            //BUG Left here and you need ToString figure out how to get 
                            int _locationMarkerCount = MapHelpers.IsMultipleMarkersAtLocation(OrderMarkers, new Location(o.Lat.DataValue.Value, o.Lon.DataValue.Value));
                            if (_locationMarkerCount > 0)
                            {
                                // Make sure there haven't been previous multi markers added to this location.
                                int _multiMarkerLocationCount = MapHelpers.IsMultipleMarkersAtLocation(OrderMarkersMulti, new Location(o.Lat.DataValue.Value, o.Lon.DataValue.Value));

                                if(_multiMarkerLocationCount == 0)
                                    OrderMarkersMulti.Add(new MarkerModel(o, Enums.MarkerType.MultiPoint));
                            }

                            OrderMarkers.Add(new MarkerModel(o, Enums.MarkerType.Transaction));
                            //else
                            //{
                            //    // We have to remove the multi image if we're down to only one record existing at a location.
                            //    // You're going to have to change the 

                            //    if(OrderMarkersMulti.Count > 0)
                            //    {
                            //        var _orderMarkerMulti =  OrderMarkersMulti.Where(om => om.Location.Latitude == o.Lat.DataValue && om.Location.Longitude == o.Lon.DataValue);

                            //        if (_orderMarkerMulti != null && _orderMarkerMulti.Count() > 1)
                            //        {
                            //            for (int i = 1; i < _orderMarkerMulti.Count(); i++)
                            //            {
                            //                OrderMarkersMulti.RemoveAt(i);
                            //            }
                                        
                            //        }
                            //    }

                            //    OrderMarkers.Add(new MarkerModel(o, Enums.MarkerType.Transaction));
                            //    //OrderMarkers.Add(new MarkerModel(o, Enums.MarkerType.MultiPoint));
                            //}
                        }

                        if (OrderMarkers.Count > 0)
                        {
                            SetMapValues();

                        }

                       
                    }));
            }
        }


        #endregion

        public void Initialize(SmartObservableCollection<OrderModel> orders)
        {
            if (orders == null || orders.Count < 1)
                throw new Exception(string.Format("{0} must be passed a collection of orders.", this.GetType().ToString()));
            else
            {
                OrderMarkers.Clear();
                OrderMarkersMulti.Clear();

                int i = 0;

                foreach (var o in orders)
                {
                    o.IsSelected = false;

                    if (o.Lat.DataValue.HasValue && o.Lon.DataValue.HasValue)
                    {
                        i++;
                        o.Index = i;
                        Orders.Add(o);
                    }
                }
                
                OrderModel _obj = new OrderModel();
                
                _obj.OrderId.DataValue = null;
                _obj.ShipName.DataValue = "Map All Orders";
                Orders.Insert(0, _obj);                

                // Set map to default levels
                ZoomLevel = _defaultZoomLevel;
                MapCenter = new Location(35.21, -80.84);
            }
        }

        public override void Cleanup()
        {            
            Messenger.Default.Unregister(this);

            Orders.Clear();
            OrderMarkers.Clear();
            OrderMarkers.Clear();
            OrderController = null;
        }

        #region Private Methods


        private void SetMapValues()
        {
            // This method attempts to manually calculate center and Zoom.  Center works good, but zoom 
            // is going to rely on size elements from the map which I haven't figured out how to do with Telerik
            //MapZoomLevelModel _zoomLevels = new MapZoomLevelModel { MaxZoom = 20, MinZoom = 1, DefaultZoom = 7 };
            //CenterAndZoomModel _czv = MapHelpers.CalculateMapCenterZoom(OrderMarkers, _zoomLevels, DistanceUnitMeasure);

            //// This way is using the calculations only from CalculateMapCenterZoom
            //MapCenter = _czv.MapCenter;
            //ZoomLevel = _czv.ZoomLevel;

            // Telerik recommended way.  However, this causes a problem when there is only one marker.
            // The zoom level goes out to 1 instead of zooming in to a specified value.
            LocationRect _locationRect = MapHelpers.SetBestView(OrderMarkers);
            Messenger.Default.Send<CustomerMapOrdersChangedEvent>(new CustomerMapOrdersChangedEvent(_locationRect, _defaultZoomLevel) );
        }


        #endregion
    }
}
