﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel.Composition;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Messaging;
using Telerik.Windows.Controls.Map;
using System.Diagnostics;
using Northwind.Common;
using System.Windows.Data;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace Northwind.Modules.Orders
{
    public partial class MapOrderView : UserControl, ICleanup
    {
        // Used in ToolTip to show data
        RadGridView _grdView;
        FrameworkElement _activeToolTipElement = null;

        public MapOrderView()
        {
            InitializeComponent();

            if (!GalaSoft.MvvmLight.ViewModelBase.IsInDesignModeStatic)
            {
                CompositionInitializer.SatisfyImports(this);
            }

            Messenger.Default.Register<CustomerMapOrdersChangedEvent>(this, SetOrderMapCenter);
        }

        [Import(typeof(MapOrderViewModel))]
        public MapOrderViewModel ViewModel
        {
            get
            {
                return this.DataContext as MapOrderViewModel;
            }
            set
            {
                this.DataContext = value;
            }
        }
        
        #region ICleanup Members

        public void Cleanup()
        {
            //call cleanup on it's ViewModel
            ((ICleanup)this.DataContext).Cleanup();

            // Cleanup itself
            Messenger.Default.Unregister(this);
        }

        #endregion

        private void Marker_MouseEnter(object sender, MouseEventArgs e)
        {            
            Marker _marker = sender as Marker;
            HideTootip(_marker);
            if (_marker != null)
            {
                // Get location of the pushpin. 
                Location location = MapLayer.GetLocation(_marker);
                List<MarkerModel> _markersAtSameLocation = new List<MarkerModel>();

                if (!location.IsEmpty)
                {
                    //string truckNames = string.Empty;
                    // Get all things are placed at the given location. 
                    //IEnumerable<object> items = this.OrderLayer.GetItemsInLocation(location);
                    foreach (object item in this.OrderLayer.Items)
                    {
                        // Access the DataContext of the pushpin 
                        MarkerModel marker = item as MarkerModel;
                        if (marker != null)
                        {
                            if (marker.Location.Latitude == location.Latitude && marker.Location.Longitude == location.Longitude)
                            {
                                _markersAtSameLocation.Add(marker);
                            }
                        }
                    }

                    if (_markersAtSameLocation.Count > 0)
                    {
                        // Show truck list info. 
                        this.ShowTooltip(_marker, _markersAtSameLocation);
                    }
                }
            }

            // So the Map won't change positions
            //e.Handled = true;
        }

        private void Marker_MouseLeave(object sender, MouseEventArgs e)
        {
            Marker _marker = sender as Marker;

            if (_marker != null)
            {
                HideTootip(_marker);
                _activeToolTipElement = null;
            }

            //e.Handled = true;
        }

        private void radMap1_CenterChanged(object sender, EventArgs e)
        {
            
        }


        /// <summary>
        /// Recommended Telerik way of setting the ZoomValue
        /// </summary>
        /// <param name="evt"></param>
        public void SetOrderMapCenter(CustomerMapOrdersChangedEvent evt)
        {
            double _distValue = 0;

            // Use Kilometers or Miles
            if (this.radMap1.DistanceUnit == DistanceUnit.Kilometer)
            {
                _distValue = 6371;
            }
            else
            {
                _distValue = 3958.75;
            }

            this.radMap1.SetView(evt.BestView);

            var _dist = (_distValue * Math.Acos(Math.Sin(evt.BestView.Southwest.Latitude / 57.2958) * Math.Sin(evt.BestView.Northeast.Latitude / 57.2958) + Math.Cos(evt.BestView.Southwest.Latitude / 57.2958) *
                        Math.Cos(evt.BestView.Northeast.Latitude / 57.2958) * Math.Cos((evt.BestView.Northeast.Longitude / 57.2958) - (evt.BestView.Southwest.Longitude / 57.2958))));

            if (_dist < .5)
            {
                this.radMap1.ZoomLevel = evt.DefaultZoomLevel;
            }
            
            // First recommended attempt at setting maps center & zoom level
            // Neither value was correct
            //if (OrderLayer.Items.Count > 0)
            //{
            //    LocationRect _zoomRect = OrderLayer.GetBestView(OrderLayer.Items);
            //    _zoomRect.MapControl = radMap1;
            //    radMap1.Center = _zoomRect.Center;
            //    radMap1.ZoomLevel = _zoomRect.ZoomLevel;
            //}
            //else
            //{
            //    radMap1.Center = new Location(39.663600, -99.1247);
            //    radMap1.ZoomLevel = 5;
                
            //}
        }

        private void ShowTooltip(FrameworkElement element, List<MarkerModel> tooltipContent)
        {
            // Need to make sure there is no way there is multiple tool tips open.
            if (_activeToolTipElement != null)
            {
                HideTootip(_activeToolTipElement);
            }

            _activeToolTipElement = element;

            if (tooltipContent != null && tooltipContent.Count > 0)
            {
                //ItemsControl _itemsCtrl = new ItemsControl();                
                //_itemsCtrl.ItemTemplate = (DataTemplate)this.Resources["ToolTipMarkersTemplate"];
                //_itemsCtrl.ItemsSource = tooltipContent;
                _grdView = new RadGridView();
                _grdView.DataLoading += new EventHandler<Telerik.Windows.Controls.GridView.GridViewDataLoadingEventArgs>(_grdView_DataLoading);
                _grdView.AutoGenerateColumns = false;
                _grdView.ShowGroupPanel = false;
                _grdView.ItemTemplate = (DataTemplate)this.Resources["ToolTipMarkersTemplate"];
                _grdView.ItemsSource = tooltipContent;
                
                //tooltipContent[0].Location.Latitude
                ToolTip tooltip = ToolTipService.GetToolTip(element) as ToolTip;

                
                if (tooltip == null)
                {
                    tooltip = new ToolTip();
                    ToolTipService.SetToolTip(element, tooltip);
                }

                if (tooltip != null)
                {
                    tooltip.Content = _grdView;
                    tooltip.IsOpen = true;
                }
            }
            else
            {
                this.HideTootip(element);
            }
        }

        void _grdView_DataLoading(object sender, Telerik.Windows.Controls.GridView.GridViewDataLoadingEventArgs e)
        {
            //List<MarkerModel> _markers = (List<MarkerModel>)e.ItemsSource;

            GridViewDataControl _dataControl = (GridViewDataControl)sender;

            GridViewDataColumn _column = new GridViewDataColumn();
            _column.Header = "Index";
            _column.DataMemberBinding = new Binding("Index");
            _dataControl.Columns.Add(_column);

            _column = new GridViewDataColumn();
            _column.Header = "Latitude";
            _column.DataMemberBinding = new Binding("Location.Latitude");
            _dataControl.Columns.Add(_column);

            _column = new GridViewDataColumn();
            _column.Header = "Longitude";
            _column.DataMemberBinding = new Binding("Location.Longitude");
            _dataControl.Columns.Add(_column);

            // Unwire event handler
            _grdView.DataLoading -= new EventHandler<Telerik.Windows.Controls.GridView.GridViewDataLoadingEventArgs>(_grdView_DataLoading);

        }

        private void HideTootip(FrameworkElement element)
        {            
            ToolTip tooltip = ToolTipService.GetToolTip(element) as ToolTip;

            if (tooltip != null)
            {
                tooltip.Content = null;
                tooltip.IsOpen = false;
            }
        }

        private void Marker_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {

        }

        private void Marker_MouseEnter_1(object sender, MouseEventArgs e)
        {

        }

    }
}
