﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel.Composition;
using GalaSoft.MvvmLight.Messaging;
using Northwind.Common;

namespace NorthwindSL
{
    public partial class App : Application
    {
        private Grid _rootGrid = new Grid();
        public static bool HasCriticalErrorWindowOpen = false;

        public App()
        {
            this.Startup += this.Application_Startup;
            this.Exit += this.Application_Exit;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            // Adding InitParams to the Resources Dictionary for use throughout the app.
            foreach (KeyValuePair<string, string> k in e.InitParams)
            {
                this.Resources.Add(k.Key, k.Value);
            }

            GlobalValues.NWSvcURI = Application.Current.Resources["NWSvcURI"].ToString();

            Bootstrapper _bs = new Bootstrapper();
            _bs.Run();


            //CompositionInitializer.SatisfyImports(this);
        }

        private void Application_Exit(object sender, EventArgs e)
        {

        }

        public virtual void InitializeRootVisual(UserControl uc)
        {
            this.RootVisual = _rootGrid;
            _rootGrid.Children.Add(uc);

            //Error Window Flow Control
            Messenger.Default.Register<CriticalErrorWindowEvent>(this, (e) => { HasCriticalErrorWindowOpen = e.HasCriticalErrorWindowOpen; });
        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            e.Handled = true;

            if (!HasCriticalErrorWindowOpen)
            {
                GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(e.ExceptionObject, string.Empty));
            }

            GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(e.ExceptionObject, string.Empty));
            // Directly instanstiating here
            //ILogService _logService = new LogService();
            //_logService.Error(e.ExceptionObject, e.ExceptionObject.GetType().ToString());
            //LogService.Error(e.ExceptionObject, e.ExceptionObject.GetType().ToString());
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }

    }
}
