﻿using System;
using System.Linq;
using Microsoft.Practices.Prism.MefExtensions;
using System.Windows;
using Microsoft.Practices.Prism.Modularity;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition.Hosting;
using Northwind.Common;
using System.Text;
using Microsoft.Practices.Prism.Regions;
using Telerik.Windows.Controls;
using Microsoft.Practices.Prism.Logging;

namespace NorthwindSL
{
    public class Bootstrapper : MefBootstrapper
    {
        private const string ModuleCatalogUri = "/NorthwindSL;component/ModulesCatalog.xaml";
        private readonly CallbackLogger callbackLogger = new CallbackLogger();
        int _modulesToLoad = 0;

        protected override CompositionContainer CreateContainer()
        {
            var _container = base.CreateContainer();
            this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(Bootstrapper).Assembly));
            this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(GlobalRegionNames).Assembly));
            CompositionHost.Initialize(_container);

            return _container;
        }

        protected override IModuleCatalog CreateModuleCatalog()
        {
            IModuleCatalog _modCat = Microsoft.Practices.Prism.Modularity.ModuleCatalog.CreateFromXaml(new Uri(ModuleCatalogUri,
                                                                                          UriKind.Relative));
            return _modCat;

        }

        protected override DependencyObject CreateShell()
        {            
            return this.Container.GetExportedValue<Shell>();
        }
        
        protected override ILoggerFacade CreateLogger()
        {
            return this.callbackLogger;
        }
        
        protected override void InitializeModules()
        {            
            //base.InitializeModules();
            IModuleManager _moduleManager = this.Container.GetExportedValue<IModuleManager>();
            _modulesToLoad = this.ModuleCatalog.Modules.Count();
            _moduleManager.LoadModuleCompleted += new EventHandler<LoadModuleCompletedEventArgs>(ModuleManager_LoadModuleCompleted);
            base.InitializeModules();
            //this.ModuleCatalog.Initialize();
            //_moduleManager.LoadModule("ScanningModule");
            //_moduleManager.LoadModule("AppSettingsModule");
            //string _modName = (this.ModuleCatalog.Modules.ElementAt(0).ModuleName);
            //_moduleManager.LoadModule(_modName);

            //_moduleManager.Run();
        }


        void ModuleManager_LoadModuleCompleted(object sender, LoadModuleCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                GlobalAppCommands.RaiseErrorMessage.Send(new AppRaiseErrorEvent(e.Error, e.Error.Message));
            }
            else
            {
                _modulesToLoad--;

                if (_modulesToLoad == 0)
                {
                    // Fire command to set the selectedTab.
                    GlobalAppCommands.AppModulesLoaded.Send(null);
                }
            }
        }

        protected override void InitializeShell()
        {
          //  base.InitializeShell();

#if SILVERLIGHT
            App _app = (App)Application.Current;
            _app.InitializeRootVisual((Shell) this.Shell);
#else
            Application.Current.MainWindow = (Shell)this.Shell;
            Application.Current.MainWindow.Show();
#endif
            IShellController _shellController = this.Container.GetExportedValue<IShellController>();
            _shellController.Initialize();
        }

        protected override RegionAdapterMappings ConfigureRegionAdapterMappings()
        {
            RegionAdapterMappings _mappings = base.ConfigureRegionAdapterMappings();
            
            _mappings.RegisterMapping(typeof(QuickAccessToolBar), this.Container.GetExportedValue<QATBRegionAdapter>());

            return _mappings;
        }


    }
}
