﻿using System;
using System.ComponentModel.Composition;
using Northwind.Common;
using System.Windows;

namespace NorthwindSL
{
    [Export(typeof(IShellController))]
    public class ShellController : IShellController
    {
        #region Private Fields

        IShellViewModel _shellViewModel;

        #endregion

        #region ctor

        [ImportingConstructor]
        public ShellController(IShellViewModel shellVieModel)
        {
            GlobalAppCommands.SetAppRootTemplate.Register(this, SetAppRootTemplate);

            // Handle Application Expections
            GlobalAppCommands.RaiseErrorMessage.Register(this, HandleRaisedErrorMessages);

            _shellViewModel = shellVieModel;

        }

        #endregion

        #region Public Properties

        [ImportMany(typeof(ILogService), AllowRecomposition = true)]
        public ILogService[] LogService { get; set; }

        [ImportMany(typeof(IPopupController), AllowRecomposition = true)]
        public IPopupController[] PopupController { get; set; }

        #endregion

        #region Public Methods

        public void Initialize()
        {

            _shellViewModel.SetAppRootTemplate(new RootShellTemplate());

        }

        #endregion
        
        #region Private Methods

        // This function is so we can pass a template as a part of an event and we can have better organization of global commands.
        private void SetAppRootTemplate(SetAppRootTemplateEvent e)
        {
            if (e != null)
            {
                _shellViewModel.SetAppRootTemplate(e.Template);
            }

        }
        
        private void HandleRaisedErrorMessages(AppRaiseErrorEvent evt)
        {
            // GlobalValues.DisplayExceptionsToClient is a setting in the Web.Config.  If set to false,
            // Any exception passed here will display in a popup window to the user.
            bool _showClientError = false;

            bool.TryParse(Application.Current.Resources["DisplayExceptionsToClient"].ToString(), out _showClientError);

            if (_showClientError)
            {
                ErrorWindow.CreateNew(evt.Ex);
            }

            //KKF:  For now, putting calling function in the ErrorMessage 

            LogService[0].Error(evt.Ex, evt.ErrorMessage);
        }
        
        #endregion
    }
}
