﻿using System;
using System.ComponentModel.Composition;
using Northwind.Common;
using System.Windows.Controls;
using GalaSoft.MvvmLight;
using MVVM.Framework;

namespace NorthwindSL
{
    [Export(typeof(IShellViewModel))]
    public class ShellViewModel : ViewModelBase, IShellViewModel, ICleanup
    {
        // A template that gives the flexibility to change up the region layout of your shell.
        // This is especially important in Silverlight where you can't change the value of 
        // the Shell Root
        private ContentPresenter _appRootTemplate;
        
        // Used to set the busy Content of the Apps global busy indicator
        private string _busyContent = string.Empty;

        // Used 
        private bool _isBusy = false;

        #region ctor

        public ShellViewModel()
        {
            _appRootTemplate = new ContentPresenter();
            
            // Register to set the Main RootVie
            GlobalAppCommands.SetAppIsBusyStatus.Register(this, SetAppIsBusyStatus);

        }

        #endregion

        #region Public Properties
        
        public ContentPresenter AppRootTemplate
        {
            get { return _appRootTemplate; }
            set
            {
                if (_appRootTemplate != value)
                {
                    _appRootTemplate = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => AppRootTemplate));
                }
            }
        }
        
        public string BusyContent
        {
            get { return _busyContent; }
            set
            {
                if (_busyContent != value)
                {
                    _busyContent = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => BusyContent));
                }
            }
        }

        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                if (_isBusy != value)
                {
                    _isBusy = value;
                    RaisePropertyChanged(ReflectionUtility.GetPropertyName(() => IsBusy));
                }
            }
        }

        [ImportMany(typeof(IPopupController), AllowRecomposition = true)]
        public IPopupController[] PopupController { get; set; }

        #endregion

        #region Private Methods


        // This will set the main pages Busy Indicator.  It will be a central place to set this
        // all throughout the application.
        private void SetAppIsBusyStatus(SetAppIsBusyStatusEvent status)
        {
            if (status == null)
            {
                IsBusy = false;
                BusyContent = string.Empty;
            }
            else
            {
                IsBusy = status.IsBusy;
                BusyContent = status.BusyContent;
            }
        }


        #endregion

        #region Public Methods

        public override void Cleanup()
        {
            base.Cleanup();

        }

        public void SetAppRootTemplate(UserControl template)
        {
            AppRootTemplate.Content = template;
        }

        #endregion
    }
}
